#ifndef CoCoA_MatrixArith_H
#define CoCoA_MatrixArith_H

//   Copyright (c)  2005  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include <cstddef>
//using std::size_t;
#include <vector>
// using std::vector

namespace CoCoA
{

  class ConstMatrix; // forward declaration -- defined in matrix.H
  class RefRingElem; // forward declaration -- defined in ring.H
  class matrix;      // forward declaration -- defined in matrix.H
  class ring;        // forward declaration -- defined in ring.H

  void DetByGauss(RefRingElem d, ConstMatrix M);
  size_t RankByGauss(ConstMatrix M);
  void InverseByGauss(matrix& lhs, ConstMatrix M);
//??? doesn't exist???  void AdjointByGauss(matrix& lhs, ConstMatrix M);
  void AdjointByInverse(matrix& lhs, ConstMatrix M);

  void GrammSchmidtRows(matrix& M);
  void GrammSchmidtRows(matrix& M, std::size_t row);

/**
    expects a matrix with entries in an ordered ring
*/
  bool IsTermOrdering(ConstMatrix M);
  matrix NewPositiveMatrix(ConstMatrix M); ///< returns a matrix with positive entries which defines an equivalent term-ordering
  bool IsPositiveGrading(ConstMatrix M, size_t GradingDim);  ///< are the first GradingDim rows of M a positive grading?

//   matrix NewBlockMatrix(ConstMatrix M11, ConstMatrix M12, ConstMatrix M21, ConstMatrix M22);
  matrix NewMatrixConcatHor(ConstMatrix M1, ConstMatrix M2);
  matrix NewMatrixConcatVer(ConstMatrix M1, ConstMatrix M2);
  matrix NewMatrixConcatDiag(ConstMatrix M1, ConstMatrix M2);
  matrix NewMatrixConcatAntiDiag(ConstMatrix M1, ConstMatrix M2);
  matrix NewMatrixMinimize(ConstMatrix M);///< the (ordering) matrix obtained by removing linearly dependent rows
  matrix NewDenseMatrixRevLex(ring R, size_t n);
  matrix NewMatrixCompleteOrd(ConstMatrix M);
  matrix NewMatrixElim(size_t NumIndets, std::vector<size_t> IndetsToElim);
  matrix NewMatrixElim(ConstMatrix GradingM, std::vector<size_t> IndetsToElim, bool IsHomog);
} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/MatrixArith.H,v 1.2 2007/10/30 17:14:12 abbott Exp $
// $Log: MatrixArith.H,v $
// Revision 1.2  2007/10/30 17:14:12  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.8  2007/03/08 18:42:06  cocoa
// Cleaned up whitespace.
//
// Revision 1.7  2007/03/07 13:07:10  cocoa
// -- minor: added comment for NewMatrixMinimize
//
// Revision 1.6  2006/12/06 17:14:12  cocoa
// -- removed #include "config.H"
//
// Revision 1.5  2006/11/24 17:27:46  cocoa
// -- reorganized includes of header files
//
// Revision 1.4  2006/10/06 14:04:15  cocoa
// Corrected position of #ifndef in header files.
// Separated CoCoA_ASSERT into assert.H from config.H;
// many minor consequential changes (have to #include assert.H).
// A little tidying of #include directives (esp. in Max's code).
//
// Revision 1.3  2006/08/17 09:39:08  cocoa
// -- added: elimination ordering matrix for non-homogeneous input
//
// Revision 1.2  2006/07/17 16:58:05  cocoa
// -- added: NewMatrixElim(size_t NumIndets, std::vector<size_t> IndetsToElim)
//
// Revision 1.1.1.1  2006/05/30 11:39:37  cocoa
// Imported files
//
// Revision 1.4  2006/04/10 13:20:43  cocoa
// -- fixed buglets for Elimination orderings
//
// Revision 1.3  2006/04/05 17:18:33  cocoa
// -- added IsPositiveGrading
//
// Revision 1.2  2006/03/27 12:21:26  cocoa
// Minor silly changes to reduce number of complaints from some compiler or other.
//
// Revision 1.1.1.1  2005/10/17 10:46:54  cocoa
// Imported files
//
// Revision 1.1.1.1  2005/05/03 15:47:30  cocoa
// Imported files
//
// Revision 1.4  2005/04/19 15:39:55  cocoa
// Matrices now use reference counts.
//
// Revision 1.3  2005/04/19 14:06:04  cocoa
// Added GPL and GFDL licence stuff.
//
// Revision 1.2  2005/03/30 17:15:14  cocoa
// Cleaned the SpecialMatrix code; a simple test compiles and
// runs fine.
//
// Revision 1.1  2005/03/11 18:40:54  cocoa
// -- first import
//

#endif
