//   Copyright (c)  2007  Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


// Source code for class DistMPolyInlPP

#include "CoCoA/DenseUPolyClean.H"
#include <algorithm>
using std::copy; // copy ctor
#include "CoCoA/error.H"
#include "CoCoA/ZZ.H"  // uses binomial

//#include <vector>
using std::vector;
//#include <memory>
using std::auto_ptr;
//#include <cstddef>
using std::size_t;
#include <algorithm>
using std::max;
//using std::swap;
#include <iostream>
using std::ostream;


namespace CoCoA
{

  DenseUPolyClean::DenseUPolyClean(const ring& R, std::size_t MinCapacity):
      myCoeffRingValue(R)
  {
    myCoeffsValue.reserve(MinCapacity);
    myDegPlus1Value = 0;
    mySizeValue = 0;
  }


  DenseUPolyClean::~DenseUPolyClean()
  {
  }


  DenseUPolyClean::DenseUPolyClean(const DenseUPolyClean& copy, std::size_t MinCapacity):
      myCoeffRingValue(copy.myCoeffRingValue),
      myDegPlus1Value(copy.myDegPlus1Value),
      mySizeValue(copy.mySizeValue)
  {
    myCoeffsValue.reserve(max(MinCapacity, copy.myDegPlus1Value));
    myCoeffsValue = copy.myCoeffsValue;
    //    copy(rawcopy.begin(), rawcopy.end(), rawlhs.back_inserter());
  }

  // ANNA: this is commented out to check whether it is really needed
//   DenseUPolyClean::DenseUPolyClean(const DenseUPolyClean& copy):
//       myCoeffRingValue(copy.myCoeffRingValue),
//       myDegPlus1Value(copy.myDegPlus1Value),
//       mySizeValue(copy.mySizeValue)
//   {
//     std::clog << "DenseUPolyClean copy ctor reserve: ";
//     myCoeffsValue.reserve(copy.myDegPlus1Value);
//     myCoeffsValue = copy.myCoeffsValue;
//     //    copy(rawcopy.begin(), rawcopy.end(), rawlhs.back_inserter());
//   }


  DenseUPolyClean& DenseUPolyClean::operator=(const DenseUPolyClean& rhs)
  {
    if (this == &rhs) return *this;
    DenseUPolyClean copy(rhs, rhs.myDegPlus1Value);
    ourSwap(*this, copy);
    return *this;
  }


  DenseUPolyClean& DenseUPolyClean::operator=(long rhs)
  { // exception safe
    if (rhs == 0)
    {
      myAssignZero();
      return *this; // to avoid needless alloc/free
    }
    myResize(1);
    myDegPlus1Value = 1;
    RingElem x(myCoeffRingValue, rhs);
    swap(myCoeffsValue[0], x);
    return *this;
  }


  DenseUPolyClean& DenseUPolyClean::operator=(const ZZ& rhs)
  { // exception safe
    //    myAssignZero();  // this would probably save memory
    if (rhs == 0)
    {
      myAssignZero();
      return *this; // to avoid needless alloc/free
    }
    myResize(1);
    myDegPlus1Value = 1;
    RingElem x(myCoeffRingValue, rhs);
    swap(myCoeffsValue[0], x);
    return *this;
  }


  bool IsCompatible(const DenseUPolyClean& f, const DenseUPolyClean& g)
  {
    return (f.myCoeffRingValue == g.myCoeffRingValue);
  }


//----------------------------------------------------------------------

  void DenseUPolyClean::ourSwap(DenseUPolyClean& f, DenseUPolyClean& g)
  {
    CoCoA_ASSERT(IsCompatible(f, g));
    std::swap(f.myCoeffsValue, g.myCoeffsValue);
    std::swap(f.myDegPlus1Value, g.myDegPlus1Value);
    std::swap(f.mySizeValue, g.mySizeValue);
  }


  void DenseUPolyClean::myAssignZero()
  {
    for (size_t i=0; i<myDegPlus1Value; ++i)
      myCoeffsValue[i] = 0;
    myDegPlus1Value = 0;
  }


  void DenseUPolyClean::myResize(std::size_t NewSize)
  {
    myCoeffsValue.resize(NewSize, zero(myCoeffRingValue));
    mySizeValue = NewSize;
  }


  void DenseUPolyClean::myResetDeg()
  {
    size_t i = myDegPlus1Value;
    for (; i!=0; --i)
      if (!IsZero(myCoeffsValue[i-1])) break;
    myDegPlus1Value = i;  // either i==0 or myCoeffsValue[i-1]!=0
  }
  

  void DenseUPolyClean::myAssignZeroCoeff(std::size_t d)
  { // exception safe?
    if (d >= myDegPlus1Value)  return;
    // d <= deg
    myCoeffsValue[d] = 0;
    if (d == myDegPlus1Value-1)
      myResetDeg();
  }


  void DenseUPolyClean::myAssignNonZeroCoeff(ConstRefRingElem c, std::size_t d)
  { // exception safe
    CoCoA_ASSERT(mySize()>=d);
    RingElem x(c);
    swap(myCoeffsValue[d], x);
    if (d >= myDegPlus1Value)
      myDegPlus1Value = d+1;
  }


  ConstRefRingElem LC(const DenseUPolyClean& f)
  {
    CoCoA_ASSERT(!IsZero(f));
    CoCoA_ASSERT(!IsZero(f.myCoeffsValue[f.myDegPlus1Value]));
    return f.myCoeffsValue[f.myDegPlus1Value];
  }


  std::size_t DenseUPolyClean::myDegPlus1() const
  {
    //    CoCoA_ASSERT(!IsZero(*this));
    return myDegPlus1Value;
  }
  

  std::size_t DenseUPolyClean::mySize() const
  {
    return mySizeValue;
  }


  void DenseUPolyClean::myNegate()
  {
    for (size_t i=0 ; i < myDegPlus1Value ; ++i)
      myCoeffRingValue->myNegate(raw(myCoeffsValue[i]), raw(myCoeffsValue[i]));   // MIGHT THROW???
  }


  void add(DenseUPolyClean& lhs, const DenseUPolyClean& f, const DenseUPolyClean& g)
  {
    CoCoA_ASSERT(IsCompatible(lhs, g) && IsCompatible(f, g));
    const ring& R = lhs.myCoeffRingValue;
    DenseUPolyClean ans(R, max(f.myCoeffsValue.capacity(), g.myCoeffsValue.capacity())); // ans.myDegPlus1Value is 0
    ans.myResize(max(g.myDegPlus1Value, f.myDegPlus1Value)+1);

    size_t i = 0;
    if (f.myDegPlus1Value >= g.myDegPlus1Value)
    {
      for (; i<g.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = f.myCoeffsValue[i] + g.myCoeffsValue[i];
      for (; i<f.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = f.myCoeffsValue[i];
    }
    else
    {
      for (; i<f.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = f.myCoeffsValue[i] + g.myCoeffsValue[i];
      for (; i<g.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = g.myCoeffsValue[i];
    }
    ans.mySizeValue = ans.myCoeffsValue.size();
    ans.myDegPlus1Value = max(f.myDegPlus1Value, g.myDegPlus1Value);
    if (f.myDegPlus1Value == g.myDegPlus1Value)
      ans.myResetDeg();
    swap(lhs, ans);
  }


  void sub(DenseUPolyClean& lhs, const DenseUPolyClean& f, const DenseUPolyClean& g)
  {
    CoCoA_ASSERT(IsCompatible(lhs, g) && IsCompatible(f, g));
    const ring& R = lhs.myCoeffRingValue;
    DenseUPolyClean ans(R, max(f.myCoeffsValue.capacity(), g.myCoeffsValue.capacity())); // ans.myDegPlus1Value is 0
    ans.myResize(max(g.myDegPlus1Value, f.myDegPlus1Value)+1);

    size_t i = 0;
    if (f.myDegPlus1Value >= g.myDegPlus1Value)
    {
      for (; i<g.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = f.myCoeffsValue[i] - g.myCoeffsValue[i];
      for (; i<f.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = f.myCoeffsValue[i];
    }
    else
    {
      for (; i<f.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = f.myCoeffsValue[i] - g.myCoeffsValue[i];
      for (; i<g.myDegPlus1Value; ++i)
        ans.myCoeffsValue[i] = - g.myCoeffsValue[i];
    }
    ans.mySizeValue = ans.myCoeffsValue.size();
    ans.myDegPlus1Value = max(f.myDegPlus1Value, g.myDegPlus1Value);
    if (f.myDegPlus1Value == g.myDegPlus1Value)
      ans.myResetDeg();
    swap(lhs, ans);
  }


  bool div(DenseUPolyClean& lhs, const DenseUPolyClean& g, const DenseUPolyClean& h)  // result is true iff quotient is exact.
  {
    if (IsZero(h)) return false; //???CoCoA_ERROR(ERR::DivByZero,"div(DMP,DMP,DMP)");
    CoCoA_ERROR(ERR::NYI, "sub(DenseUPolyClean&, const DenseUPolyClean&, const DenseUPolyClean&)");
    return true;
  }


  void DenseUPolyClean::myAddMul(ConstRefRingElem c, std::size_t d, const DenseUPolyClean& g)
  {
    myResize(max(myDegPlus1Value, g.myDegPlus1Value + d));

    if (IsOne(c)) // special case for Hilbert Poincare Series
      for (size_t i=0; i<g.myDegPlus1Value; ++i)
      {
        if (!IsZero(g.myCoeffsValue[i]))
          myCoeffsValue[i+d] += g.myCoeffsValue[i];
      }
    else
      for (size_t i=0; i<g.myDegPlus1Value; ++i)
      {
        if (!IsZero(g.myCoeffsValue[i]))
          myCoeffsValue[i+d] += c * g.myCoeffsValue[i];
      }
    myDegPlus1Value = max(myDegPlus1Value, g.myDegPlus1Value+d);
    if (myDegPlus1Value == g.myDegPlus1Value+d)
      myResetDeg();
    if (!IsIntegralDomain(myCoeffRingValue))
      myResetDeg();
  }


  void DenseUPolyClean::myMulByCoeff(ConstRefRingElem c)
  {
    for (size_t i=0 ; i < myDegPlus1Value ; ++i)
      myCoeffsValue[i] *= c;
    if (!IsIntegralDomain(myCoeffRingValue))
      myResetDeg();
  }


  void DenseUPolyClean::myMulByXExp(int n)
  { // EXCEPTION SAFE
    myResize(myDegPlus1Value + n); // EXCEPTION SAFE ???
    for (size_t i=myDegPlus1Value; i!=0  ; )
    {
      --i;
      swap(myCoeffsValue[i+n], myCoeffsValue[i]);
    }
    myDegPlus1Value = myDegPlus1Value + n;
  }


  void DenseUPolyClean::myMulBy1MinusXExp(int n)
  { // NOT EXCEPTION SAFE
    myResize(myDegPlus1Value + n);
    for (size_t i=myDegPlus1Value; i!=0  ; )
    {
      --i;
      myCoeffsValue[i+n] -= myCoeffsValue[i];
    }
    myDegPlus1Value = myDegPlus1Value + n;
  }


  void output(ostream& out, const DenseUPolyClean& f)  // for debugging only
  {
    if (IsZero(f)) { out << "0"; return; }
    for (size_t i=0; i<f.myDegPlus1Value; ++i)
      out << " +(" << f.myCoeffsValue[i] << ")*indet^" << i;
  }


  bool IsZero(const DenseUPolyClean& f)
  {
    return f.myDegPlus1Value == 0;
  }


  bool IsMonomial(const DenseUPolyClean& f)
  {
    CoCoA_ERROR(ERR::NYI, "IsMonomial(const DenseUPolyClean&)");
    return true;
  }


  bool IsEqual(const DenseUPolyClean& f, const DenseUPolyClean& g)
  {
    CoCoA_ASSERT(IsCompatible(f, g));
    if (&f == &g) return true;
    if (f.myDegPlus1Value != g.myDegPlus1Value) return false;
    for (size_t i=0; i < f.myDegPlus1Value; ++i)
      if (f.myCoeffsValue[i] != g.myCoeffsValue[i]) return false;
    return true;
  }


  void deriv(DenseUPolyClean& lhs, const DenseUPolyClean& f)
  {
    CoCoA_ERROR(ERR::NYI, "deriv(DenseUPolyClean&, const DenseUPolyClean&)");
  }

} // end of namespace CoCoA



// RCS header/log in the next few lines
// $Header: /Volumes/Home/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/DenseUPolyClean.C,v 1.8 2007/11/29 17:43:58 bigatti Exp $
// $Log: DenseUPolyClean.C,v $
// Revision 1.8  2007/11/29 17:43:58  bigatti
// -- fixed  operator=  which used automatic copy-ctor
// -- copy-ctor declared but not defined for checking
//
// Revision 1.7  2007/10/30 17:14:08  abbott
// Changed licence from GPL-2 only to GPL-3 or later.
// New version for such an important change.
//
// Revision 1.6  2007/10/19 10:04:23  bigatti
// -- RingDenseUPolyClean now allow to specify the MinCapacity for all
//    coeff vectors (to avoid too many reallocations)
//
// Revision 1.5  2007/10/19 09:03:46  bigatti
// -- the two contructors reserve 150 slots for coefficients.
//
// Revision 1.4  2007/10/15 13:43:24  bigatti
// -- improved myAddMul for c==1 (called from RecPoincare)
//
// Revision 1.3  2007/10/15 12:40:31  bigatti
// -- new implementation for myAddMul
// -- new: coeffs between deg+1 and size are now guaranteed to be 0
//
// Revision 1.2  2007/10/10 14:02:37  bigatti
// -- added myMulBy1MinusXExp
// -- fixed a few little bugs
//
// Revision 1.1  2007/10/05 15:28:56  bigatti
// -- added abstract class DenseUPolyRing for representing dense
//    univariate polynomials
// -- added concrete class RingDenseUPolyClean, the cleanest
//    implementation
// -- just for testing, still horribly inefficient and incomplete
//
